@@ -16,10 +16,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
+#include <sys/types.h>
+#include <sys/wait.h>
 #include "cdplay.h"
 #include "grip.h"
 #include "config.h"
@@ -35,6 +37,10 @@
 static void ShutDownCB(GtkWidget *widget,gpointer data);
 static void DiscDBToggle(GtkWidget *widget,gpointer data);
 static void DoLookup(void *data);
+static void DoNextDiscDBEntry(GtkWidget *widget, gpointer data);
+static void DiscDBReadDisc(GripInfo *ginfo, DiscDBServer *server,
+		        DiscDBQuery *query, int query_idx);
+static void DoPrevDiscDBEntry(GtkWidget *widget, gpointer data);
 static void SetCurrentTrack(GripInfo *ginfo,int track);
 static void ToggleChecked(GripGUI *uinfo,int track);
 static void ClickColumn(GtkTreeViewColumn *column,gpointer data);
@@ -80,8 +86,8 @@ static void DiscDBToggle(GtkWidget *widget,gpointer data)
   }
   else {
     if(ginfo->ripping_a_disc) {
-      gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                        _("Cannot do lookup while ripping."));
+      DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+                 _("Cannot do lookup while ripping."));
 
       return;
     }
@@ -146,21 +152,16 @@ void LookupDisc(GripInfo *ginfo,gboolean manual)
 static void DoLookup(void *data)
 {
   GripInfo *ginfo;
-  gboolean discdb_found=FALSE;
 
   ginfo=(GripInfo *)data;
 
   if(!DiscDBLookupDisc(ginfo,&(ginfo->dbserver))) {
     if(*(ginfo->dbserver2.name)) {
       if(DiscDBLookupDisc(ginfo,&(ginfo->dbserver2))) {
-        discdb_found=TRUE;
         ginfo->ask_submit=TRUE;
       }
     }
   }
-  else {
-    discdb_found=TRUE;
-  }
 
   if(ginfo->ddata.data_id3genre==-1)
     ginfo->ddata.data_id3genre=DiscDB2ID3(ginfo->ddata.data_genre);
@@ -173,7 +174,6 @@ gboolean DiscDBLookupDisc(GripInfo *ginfo,DiscDBServer *server)
 {
   DiscDBHello hello;
   DiscDBQuery query;
-  DiscDBEntry entry;
   gboolean success=FALSE;
   DiscInfo *disc;
   DiscData *ddata;
@@ -181,15 +181,6 @@ gboolean DiscDBLookupDisc(GripInfo *ginfo,DiscDBServer *server)
   disc=&(ginfo->disc);
   ddata=&(ginfo->ddata);
 
-  if(server->use_proxy)
-    LogStatus(ginfo,_("Querying %s (through %s) for disc %02x.\n"),
-	      server->name,
-	      server->proxy->name,
-	      DiscDBDiscid(disc));
-  else
-    LogStatus(ginfo,_("Querying %s for disc %02x.\n"),server->name,
-	      DiscDBDiscid(disc));
-
   strncpy(hello.hello_program,"Grip",256);
   strncpy(hello.hello_version,VERSION,256);
 
@@ -201,28 +192,14 @@ gboolean DiscDBLookupDisc(GripInfo *ginfo,DiscDBServer *server)
   if(!DiscDBDoQuery(disc,server,&hello,&query)) {
     ginfo->update_required=TRUE;
   } else {
+    ddata->data_query = query;
+	ddata->data_query_idx = 0;
     switch(query.query_match) {
     case MATCH_INEXACT:
     case MATCH_EXACT:
-      LogStatus(ginfo,_("Match for \"%s / %s\"\nDownloading data...\n"),
-		query.query_list[0].list_artist,
-                query.query_list[0].list_artist);
-
-      entry.entry_genre = query.query_list[0].list_genre;
-      entry.entry_id = query.query_list[0].list_id;
-      DiscDBRead(disc,server,&hello,&entry,ddata,ginfo->discdb_encoding);
-
-      Debug(_("Done\n"));
-      success=TRUE;
-		
-      if(DiscDBWriteDiscData(disc,ddata,NULL,TRUE,FALSE,"utf-8")<0)
-	g_print(_("Error saving disc data\n"));
-
-      ginfo->update_required=TRUE;
-      ginfo->is_new_disc=TRUE;
+		DiscDBReadDisc(ginfo, server, &query, 0);
       break;
     case MATCH_NOMATCH:
-      LogStatus(ginfo,_("No match\n"));
       break;
     }
   }
@@ -230,6 +207,40 @@ gboolean DiscDBLookupDisc(GripInfo *ginfo,DiscDBServer *server)
   return success;
 }
 
+void DiscDBReadDisc(GripInfo *ginfo, DiscDBServer *server,
+		DiscDBQuery *query, int query_idx)
+{
+  DiscDBHello hello;
+  DiscDBEntry entry;
+  DiscInfo *disc;
+  DiscData *ddata;
+
+  disc=&(ginfo->disc);
+  ddata=&(ginfo->ddata);
+
+  strncpy(hello.hello_program,"Grip",256);
+  strncpy(hello.hello_version,VERSION,256);
+
+  if(ginfo->db_use_freedb)
+    hello.proto_version=6;
+  else
+    hello.proto_version=5;
+
+  entry.entry_genre = query->query_list[query_idx].list_genre;
+  entry.entry_id = query->query_list[query_idx].list_id;
+
+  DiscDBRead(disc,server,&hello,&entry,ddata,ginfo->discdb_encoding);
+
+  Debug(_("Done\n"));
+
+  if(DiscDBWriteDiscData(disc,ddata,NULL,TRUE,FALSE,"utf-8")<0)
+	  g_print(_("Error saving disc data\n"));
+
+  ginfo->update_required=TRUE;
+  ginfo->is_new_disc=TRUE;
+
+}
+
 int GetLengthRipWidth(GripInfo *ginfo)
 {
   GtkWidget *track_list;
@@ -270,7 +281,6 @@ void ResizeTrackList(GripInfo *ginfo)
   GtkWidget *track_list;
   GtkTreeViewColumn *column;
   int tot_width=0;
-  PangoLayout *layout;
 
   track_list=ginfo->gui_info.track_list;
 
@@ -309,6 +319,27 @@ void MakeTrackPage(GripInfo *ginfo)
   gtk_box_pack_start(GTK_BOX(vbox),uinfo->disc_artist_label,FALSE,FALSE,0);
   gtk_widget_show(uinfo->disc_artist_label);
 
+  uinfo->dbinfo_hbox=gtk_hbox_new(FALSE,0);
+
+  uinfo->dbinfo_back_button=gtk_button_new_from_stock(GTK_STOCK_GO_BACK);
+  gtk_tooltips_set_tip(MakeToolTip(),uinfo->dbinfo_back_button,
+		  _("Select previous CDDB entry."),NULL);
+  gtk_signal_connect(GTK_OBJECT(uinfo->dbinfo_back_button),"clicked",
+		  GTK_SIGNAL_FUNC(DoPrevDiscDBEntry),(gpointer)ginfo);
+  gtk_box_pack_start(GTK_BOX(uinfo->dbinfo_hbox),uinfo->dbinfo_back_button,FALSE,FALSE,0);
+  gtk_widget_show(uinfo->dbinfo_back_button);
+
+  uinfo->dbinfo_forward_button=gtk_button_new_from_stock(GTK_STOCK_GO_FORWARD);
+  gtk_tooltips_set_tip(MakeToolTip(),uinfo->dbinfo_forward_button,
+		  _("Select next CDDB entry."),NULL);
+  gtk_signal_connect(GTK_OBJECT(uinfo->dbinfo_forward_button),"clicked",
+		  GTK_SIGNAL_FUNC(DoNextDiscDBEntry),(gpointer)ginfo);
+  gtk_box_pack_start(GTK_BOX(uinfo->dbinfo_hbox),uinfo->dbinfo_forward_button,FALSE,FALSE,0);
+  gtk_widget_show(uinfo->dbinfo_forward_button);
+
+  gtk_box_pack_start(GTK_BOX(vbox),uinfo->dbinfo_hbox,FALSE,FALSE,0);
+  gtk_widget_show(uinfo->dbinfo_hbox);
+
   uinfo->track_list_store=gtk_list_store_new(TRACKLIST_N_COLUMNS,
 					     G_TYPE_STRING,
 					     G_TYPE_STRING,
@@ -400,6 +431,60 @@ void MakeTrackPage(GripInfo *ginfo)
   gtk_widget_show(vbox);
 }
 
+void DoNextDiscDBEntry(GtkWidget *widget, gpointer data)
+{
+	GripInfo *ginfo;
+	ginfo=(GripInfo *)data;
+
+	DiscData *ddata = &(ginfo->ddata);
+
+	Debug(_("DoNextDiscDBEntry.\n"));
+
+	if (ddata == NULL) {
+	Debug(_("No disc data. Returning.\n"));
+		return;
+	}
+
+	if (ddata->data_query_idx >= (ddata->data_query.query_matches - 1)) {
+	Debug(_("Already at last index. Returning.\n"));
+		return;
+	}
+
+	ddata->data_query_idx++;
+
+	DiscDBReadDisc(ginfo, &(ginfo->dbserver), &(ddata->data_query),
+			ddata->data_query_idx);
+
+	Debug(_("Done.\n"));
+}
+
+void DoPrevDiscDBEntry(GtkWidget *widget, gpointer data)
+{
+	GripInfo *ginfo;
+	ginfo=(GripInfo *)data;
+
+	DiscData *ddata = &(ginfo->ddata);
+
+	Debug(_("DoPrevDiscDBEntry.\n"));
+
+	if (ddata == NULL) {
+	Debug(_("No disc data. Returning.\n"));
+		return;
+	}
+
+	if (ddata->data_query_idx <= 0) {
+	Debug(_("Already at first index. Returning.\n"));
+		return;
+	}
+
+	ddata->data_query_idx--;
+
+	DiscDBReadDisc(ginfo, &(ginfo->dbserver), &(ddata->data_query),
+			ddata->data_query_idx);
+
+	Debug(_("Done.\n"));
+}
+
 void SetCurrentTrackIndex(GripInfo *ginfo,int track)
 {
   /* Looks up the track of index track in the program */
@@ -415,7 +500,6 @@ static void SetCurrentTrack(GripInfo *ginfo,int track)
 {
   char buf[256];
   int tracklen;
-  const gchar *st,*st2;
 
   GripGUI *uinfo;
 
@@ -596,10 +680,8 @@ static void SelectionChanged(GtkTreeSelection *selection,gpointer data)
   GtkTreeModel *model;
   int row=-1;
   GripInfo *ginfo;
-  GripGUI *uinfo;
  
   ginfo=(GripInfo *)data;
-  uinfo=&(ginfo->gui_info);
   
   if(gtk_tree_selection_get_selected(selection,&model,&iter)) {
     gtk_tree_model_get(model,&iter,TRACKLIST_NUM_COL,&row,-1);
@@ -626,8 +708,8 @@ static void PlaylistChanged(GtkWindow *window,GtkWidget *widget,gpointer data)
 
   if(DiscDBWriteDiscData(&(ginfo->disc),&(ginfo->ddata),NULL,TRUE,FALSE,
                          "utf-8")<0)
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Error saving disc data."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Error saving disc data."));
 }
 
 static void ToggleLoop(GtkWidget *widget,gpointer data)
@@ -1166,8 +1248,8 @@ static void FastFwdCB(GtkWidget *widget,gpointer data)
   ginfo=(GripInfo *)data;
 
   if(ginfo->ripping_a_disc) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Cannot fast forward while ripping."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Cannot fast forward while ripping."));
 
     return;
   }
@@ -1197,8 +1279,8 @@ static void RewindCB(GtkWidget *widget,gpointer data)
   ginfo=(GripInfo *)data;
 
   if(ginfo->ripping_a_disc) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Cannot rewind while ripping."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Cannot rewind while ripping."));
 
     return;
   }
@@ -1228,8 +1310,8 @@ static void NextDisc(GtkWidget *widget,gpointer data)
   ginfo=(GripInfo *)data;
 
   if(ginfo->ripping_a_disc) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Cannot switch discs while ripping."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Cannot switch discs while ripping."));
 
     return;
   }
@@ -1247,11 +1329,9 @@ void EjectDisc(GtkWidget *widget,gpointer data)
 
   ginfo=(GripInfo *)data;
 
-  LogStatus(ginfo,_("Eject disc\n"));
-
   if(ginfo->ripping_a_disc) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Cannot eject while ripping."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Cannot eject while ripping."));
 
     return;
   }
@@ -1327,8 +1407,8 @@ void PlayTrackCB(GtkWidget *widget,gpointer data)
   disc=&(ginfo->disc);
 
   if(ginfo->ripping_a_disc) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Cannot play while ripping."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Cannot play while ripping."));
    
     return;
   }
@@ -1395,8 +1475,8 @@ void NextTrackCB(GtkWidget *widget,gpointer data)
 void NextTrack(GripInfo *ginfo)
 {
   if(ginfo->ripping_a_disc) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Cannot switch tracks while ripping."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Cannot switch tracks while ripping."));
     return;
   }
   
@@ -1426,8 +1506,8 @@ void PrevTrackCB(GtkWidget *widget,gpointer data)
 static void PrevTrack(GripInfo *ginfo)
 {
   if(ginfo->ripping_a_disc) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Cannot switch tracks while ripping."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Cannot switch tracks while ripping."));
     return;
   }
 
@@ -1829,8 +1909,8 @@ void UpdateTracks(GripInfo *ginfo)
     if(*(ginfo->cdupdate)) {
       FillInTrackInfo(ginfo,0,&enc_track);
 
-      TranslateAndLaunch(ginfo->cdupdate,TranslateSwitch,&enc_track,
-			 FALSE,&(ginfo->sprefs),CloseStuff,(void *)ginfo);
+      TranslateAndLaunch(ginfo->cdupdate, TranslateSwitch, &enc_track,
+                         FALSE, &(ginfo->sprefs));
     }
   }
   else {
@@ -1884,13 +1964,11 @@ void UpdateTracks(GripInfo *ginfo)
   }
 
   if(ginfo->ask_submit) {
-    gnome_app_ok_cancel_modal
-      ((GnomeApp *)uinfo->app,
-       _("This disc has been found on your secondary server,\n"
-       "but not on your primary server.\n\n"
-       "Do you wish to submit this disc information?"),
-       SubmitEntry,(gpointer)ginfo);
-    
+    SubmitEntry(ConfirmDialog((GtkWindow *)uinfo->app,
+                              _("This disc has been found on your secondary server,\n"
+                                "but not on your primary server.\n\n"
+                                "Do you wish to submit this disc information?")),
+                ginfo);
     ginfo->ask_submit=FALSE;
   }
 
@@ -1901,6 +1979,7 @@ void SubmitEntry(gint reply,gpointer data)
 {
   GripInfo *ginfo;
   int fd;
+  int i;
   FILE *efp;
   char mailcmd[256];
   char filename[256];
@@ -1913,8 +1992,8 @@ void SubmitEntry(gint reply,gpointer data)
   fd = mkstemp(filename);
 
   if(fd == -1) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Error: Unable to create temporary file."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Error: Unable to create temporary file."));
     return;
   }
 
@@ -1922,8 +2001,8 @@ void SubmitEntry(gint reply,gpointer data)
 
   if(!efp) {
     close(fd);
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Error: Unable to create temporary file."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Error: Unable to create temporary file."));
   }
   else {
     fprintf(efp,"To: %s\nFrom: %s\nSubject: cddb %s %02x\n",
@@ -1940,8 +2019,8 @@ void SubmitEntry(gint reply,gpointer data)
     if(DiscDBWriteDiscData(&(ginfo->disc),&(ginfo->ddata),efp,FALSE,
 			   ginfo->db_use_freedb,ginfo->db_use_freedb?
 			   "UTF-8":ginfo->discdb_encoding)<0) {
-      gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                        _("Error: Unable to write disc data."));
+      DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+                 _("Error: Unable to write disc data."));
       fclose(efp);
     }
     else {
@@ -1952,7 +2031,10 @@ void SubmitEntry(gint reply,gpointer data)
 
       Debug(_("Mailing entry to %s\n"),ginfo->discdb_submit_email);
 
-      system(mailcmd);
+      i=system(mailcmd);
+      if (i!=0 && WIFEXITED(i)!=0)
+        DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+                   _("Error: Unable to execute mail command."));
 
       remove(filename);
     }
