@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -52,9 +52,19 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
   GtkObject *adj;
   ID3Genre *id3_genre;
   gint id3_genre_count;
+  int i;
   int len;
+  int newlen;
   int dub_size;
   PangoLayout *layout;
+  char *labels[] = {
+    _("Disc title"),
+    _("Disc artist"),
+    _("ID3 genre:"),
+    _("Disc year"),
+    _("Track name"),
+    _("Track artist")
+  };
 
   uinfo=&(ginfo->gui_info);
 
@@ -64,22 +74,20 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
 
   hbox=gtk_hbox_new(FALSE,3);
 
-  label=gtk_label_new(_("Disc title"));
+  label=gtk_label_new(labels[0]);
 
-  /* This should be the longest string in the track edit section */
-
-
-
-  layout=gtk_widget_create_pango_layout(GTK_WIDGET(label),
-					_("Track name"));
-
-
-  pango_layout_get_size(layout,&len,NULL);
+  len=0;
+  for(i=0;i<6;i++) {
+    layout=gtk_widget_create_pango_layout(GTK_WIDGET(label),labels[i]);
+    pango_layout_get_size(layout,&newlen,NULL);
+    if (newlen>len) {
+      len=newlen;
+    }
+    g_object_unref(layout);
+  }
 
   len/=PANGO_SCALE;
 
-  g_object_unref(layout);
-
   layout=gtk_widget_create_pango_layout(GTK_WIDGET(label),
 					_("W"));
 
@@ -107,7 +115,7 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
 
   hbox=gtk_hbox_new(FALSE,3);
 
-  label=gtk_label_new(_("Disc artist"));
+  label=gtk_label_new(labels[1]);
   gtk_widget_set_usize(label,len,0);
   gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
   gtk_widget_show(label);
@@ -124,7 +132,7 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
 
   hbox=gtk_hbox_new(FALSE,3);
 
-  label=gtk_label_new(_("ID3 genre:"));
+  label=gtk_label_new(labels[2]);
   gtk_widget_set_usize(label,len,0);
   gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
   gtk_widget_show(label);
@@ -135,7 +143,7 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
       id3_genre_count++) {
     item = gtk_list_item_new_with_label(id3_genre->name);
     gtk_object_set_user_data(GTK_OBJECT(item),
-			     (gpointer)(id3_genre->num));
+			     GINT_TO_POINTER(id3_genre->num));
     uinfo->id3_genre_item_list=g_list_append(uinfo->id3_genre_item_list,item);
     gtk_signal_connect(GTK_OBJECT(item),"select",
 		       GTK_SIGNAL_FUNC(ID3GenreChanged),
@@ -155,7 +163,7 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
 
   hbox=gtk_hbox_new(FALSE,3);
 
-  label=gtk_label_new(_("Disc year"));
+  label=gtk_label_new(labels[3]);
   gtk_widget_set_usize(label,len,0);
   gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
   gtk_widget_show(label);
@@ -173,7 +181,7 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
 
   hbox=gtk_hbox_new(FALSE,3);
 
-  label=gtk_label_new(_("Track name"));
+  label=gtk_label_new(labels[4]);
   gtk_widget_set_usize(label,len,0);
   gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
   gtk_widget_show(label);
@@ -193,7 +201,7 @@ GtkWidget *MakeEditBox(GripInfo *ginfo)
 
   hbox=gtk_hbox_new(FALSE,3);
 
-  label=gtk_label_new(_("Track artist"));
+  label=gtk_label_new(labels[5]);
   gtk_widget_set_usize(label,len,0);
   gtk_box_pack_start(GTK_BOX(hbox),label,FALSE,FALSE,0);
   gtk_widget_show(label);
@@ -386,11 +394,11 @@ static void SaveDiscInfo(GtkWidget *widget,gpointer data)
   if(ginfo->have_disc) {
     if(DiscDBWriteDiscData(&(ginfo->disc),&(ginfo->ddata),NULL,TRUE,FALSE,
                            "utf-8")<0)
-      gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                        _("Error saving disc data."));
+      DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+                 _("Error saving disc data."));
   }
-  else gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                         _("No disc present."));
+  else DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+                  _("No disc present."));
 }
 
 static void TitleEditChanged(GtkWidget *widget,gpointer data)
@@ -481,29 +489,33 @@ static void ID3GenreChanged(GtkWidget *widget,gpointer data)
 
   ginfo=(GripInfo *)data;
 
-  ginfo->ddata.data_id3genre=(int)gtk_object_get_user_data(GTK_OBJECT(widget));
+  ginfo->ddata.data_id3genre=GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)));
   /*  ginfo->ddata.data_genre=ID32DiscDB(ginfo->ddata.data_id3genre);*/
 }
 
 static void SeparateFields(char *buf,char *field1,char *field2,char *sep)
 {
   char *tmp;
-  char spare[80];
+  char f1[129];
 
   tmp=strtok(buf,sep);
 
   if(!tmp) return;
 
-  strncpy(spare,g_strstrip(tmp),80);
+  strncpy(f1,g_strstrip(tmp),128);
+  f1[128] = 0;
 
   tmp=strtok(NULL,"");
 
   if(tmp) {
-    strncpy(field2,g_strstrip(tmp),80);
+    char f2[129];
+    strncpy(f2,g_strstrip(tmp),128);
+    f2[128] = 0;
+    strcpy(field2,f2);
   }
   else *field2='\0';
 
-  strcpy(field1,spare);
+  strcpy(field1,f1);
 }
 
 static void SplitTitleArtist(GtkWidget *widget,gpointer data)
@@ -513,7 +525,7 @@ static void SplitTitleArtist(GtkWidget *widget,gpointer data)
   int mode;
 
   ginfo=(GripInfo *)data;
-  mode=(int)gtk_object_get_user_data(GTK_OBJECT(widget));
+  mode=GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)));
 
   for(track=0;track<ginfo->disc.num_tracks;track++) {
     if(mode==0)
@@ -539,30 +551,30 @@ static void SubmitEntryCB(GtkWidget *widget,gpointer data)
   ginfo=(GripInfo *)data;
 
   if(!ginfo->have_disc) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Cannot submit. No disc is present."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Cannot submit. No disc is present."));
 
     return;
   }
 
   if(!ginfo->ddata.data_genre) {
-    /*    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-          _("Submission requires a genre other than 'unknown'."));*/
+    /* DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+                  _("Submission requires a genre other than 'unknown'.")); */
     GetDiscDBGenre(ginfo);
 
     return;
   }
 
   if(!*ginfo->ddata.data_title) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("You must enter a disc title."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("You must enter a disc title."));
 
     return;
   }
 
   if(!*ginfo->ddata.data_artist) {
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("You must enter a disc artist."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("You must enter a disc artist."));
     
     return;
   }
@@ -570,19 +582,18 @@ static void SubmitEntryCB(GtkWidget *widget,gpointer data)
   len=strlen(ginfo->discdb_submit_email);
 
   if(!strncasecmp(ginfo->discdb_submit_email+(len-9),".cddb.com",9))
-    gnome_app_ok_cancel_modal
-      ((GnomeApp *)ginfo->gui_info.app,
-       _("You are about to submit this disc information\n"
-       "to a commercial CDDB server, which will then\n"
-       "own the data that you submit. These servers make\n"
-       "a profit out of your effort. We suggest that you\n"
-       "support free servers instead.\n\nContinue?"),
-       (GnomeReplyCallback)SubmitEntry,(gpointer)ginfo);
+    SubmitEntry(ConfirmDialog((GtkWindow *)ginfo->gui_info.app,
+                              _("You are about to submit this disc information\n"
+                                "to a commercial CDDB server, which will then\n"
+                                "own the data that you submit. These servers make\n"
+                                "a profit out of your effort. We suggest that you\n"
+                                "support free servers instead.\n\nContinue?")),
+                ginfo);
   else
-    gnome_app_ok_cancel_modal
-      ((GnomeApp *)ginfo->gui_info.app,
-       _("You are about to submit this\ndisc information via email.\n\n"
-       "Continue?"),(GnomeReplyCallback)SubmitEntry,(gpointer)ginfo);
+    SubmitEntry(ConfirmDialog((GtkWindow *)ginfo->gui_info.app,
+                              _("You are about to submit this\ndisc information via email.\n\n"
+                                "Continue?")),
+                ginfo);
 }
 
 /* Make the user pick a DiscDB genre on submit*/
@@ -619,7 +630,7 @@ static void GetDiscDBGenre(GripInfo *ginfo)
   for(genre=0;genre<12;genre++) {
     item=gtk_list_item_new_with_label(DiscDBGenre(genre));
     gtk_object_set_user_data(GTK_OBJECT(item),
-			     (gpointer)genre);
+			     GINT_TO_POINTER(genre));
     gtk_signal_connect(GTK_OBJECT(item), "select",
 		       GTK_SIGNAL_FUNC(DiscDBGenreChanged),(gpointer)ginfo);
     gtk_container_add(GTK_CONTAINER(GTK_COMBO(genre_combo)->list),item);
@@ -664,5 +675,5 @@ static void DiscDBGenreChanged(GtkWidget *widget,gpointer data)
 
   ginfo=(GripInfo *)data;
 
-  ginfo->ddata.data_genre=(int)gtk_object_get_user_data(GTK_OBJECT(widget));
+  ginfo->ddata.data_genre=GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(widget)));
 }
