@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -26,11 +26,11 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/param.h>
-#include <gdk/gdkx.h>
-#include <X11/Xlib.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 #include <time.h>
+#include <gdk/gdkx.h> /* GDK_DISPLAY() */
 #include "grip.h"
-#include <libgnomeui/gnome-window-icon.h>
 #include "discdb.h"
 #include "cdplay.h"
 #include "discedit.h"
@@ -43,14 +43,14 @@
 #include "tray.h"
 
 static void ReallyDie(gint reply,gpointer data);
-static void MakeStatusPage(GripInfo *ginfo);
 static void DoHelp(GtkWidget *widget,gpointer data);
 static void MakeHelpPage(GripInfo *ginfo);
 static void MakeAboutPage(GripGUI *uinfo);
 static void MakeStyles(GripGUI *uinfo);
-static void Homepage(void);
+static void Homepage(GtkWidget *widget,gpointer data);
 static void LoadImages(GripGUI *uinfo);
 static void DoLoadConfig(GripInfo *ginfo);
+void LoadProxyConfig(GripInfo *ginfo, char* proxy_env);
 void DoSaveConfig(GripInfo *ginfo);
 
 #define BASE_CFG_ENTRIES \
@@ -169,7 +169,7 @@ gboolean AppWindowStateCB(GtkWidget *widget, GdkEventWindowState *event, gpointe
   return FALSE;
 }
 
-GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
+GtkWidget *GripNew(char *device,char *scsi_device,
 		   char *config_filename,
 		   gboolean force_small,
 		   gboolean local_mode,gboolean no_redirect)
@@ -179,20 +179,21 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
   GripGUI *uinfo;
   int major,minor,point;
   char buf[256];
+  GList icon_list = {NULL, NULL, NULL};
 
-  gnome_window_icon_set_default_from_file(GNOME_ICONDIR"/gripicon.png");
+  icon_list.data=gdk_pixbuf_new_from_file(GNOME_ICONDIR"/grip.png", NULL);
+  gtk_window_set_default_icon_list(&icon_list);
+
+  app = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_title(GTK_WINDOW(app), _("Grip"));
+  gtk_window_set_screen(GTK_WINDOW(app), gdk_screen_get_default());
 
-  app=gnome_app_new(PACKAGE,_("Grip"));
- 
   ginfo=g_new0(GripInfo,1);
 
   gtk_object_set_user_data(GTK_OBJECT(app),(gpointer)ginfo);
 
   uinfo=&(ginfo->gui_info);
   uinfo->app=app;
-  uinfo->status_window=NULL;
-  uinfo->rip_status_window=NULL;
-  uinfo->encode_status_window=NULL;
   uinfo->track_list=NULL;
 
   uinfo->win_width=WINWIDTH;
@@ -201,26 +202,6 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
   uinfo->win_width_min=MIN_WINWIDTH;
   uinfo->win_height_min=MIN_WINHEIGHT;
 
-  /*  if(geometry != NULL) {
-    gint x,y,w,h;
-    
-    if(gnome_parse_geometry(geometry, 
-			    &x,&y,&w,&h)) {
-      if(x != -1) {
-	gtk_widget_set_uposition(app,x,y);
-      }
-      
-      if(w != -1) {
-        uinfo->win_width=w;
-        uinfo->win_height=h;
-      }
-    }
-    else {
-      g_error(_("Could not parse geometry string `%s'"), geometry);
-    }
-  }
-  */
-
   if(config_filename && *config_filename)
     g_snprintf(ginfo->config_filename,256,"%s",config_filename);
   else {
@@ -241,7 +222,7 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
   if(!CDInitDevice(ginfo->cd_device,&(ginfo->disc))) {
     sprintf(buf,_("Error: Unable to initialize [%s]\n"),ginfo->cd_device);
 
-    DisplayMsg(buf);
+    DisplayMsg(GTK_MESSAGE_INFO, NULL, buf);
   }
 
   CDStat(&(ginfo->disc),TRUE);
@@ -285,7 +266,6 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
   MakeTrackPage(ginfo);
   MakeRipPage(ginfo);
   MakeConfigPage(ginfo);
-  MakeStatusPage(ginfo);
   MakeHelpPage(ginfo);
   MakeAboutPage(uinfo);
   ginfo->tray_icon_made = FALSE;
@@ -311,8 +291,9 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
     gtk_box_pack_start(GTK_BOX(uinfo->winbox),uinfo->controls,FALSE,FALSE,0);
   gtk_widget_show(uinfo->controls);
   
-  gnome_app_set_contents(GNOME_APP(app),uinfo->winbox);
+  gtk_container_add(GTK_CONTAINER(app), uinfo->winbox);
   gtk_widget_show(uinfo->winbox);
+  gtk_widget_show(app);
 
   CheckNewDisc(ginfo,FALSE);
 
@@ -324,15 +305,13 @@ GtkWidget *GripNew(const gchar* geometry,char *device,char *scsi_device,
 
     /* Check if we have a dev release */
     /*if(minor%2) {
-      gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                        _("This is a development version of Grip. If you encounter problems, you are encouraged to revert to the latest stable version."));
+      DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+                 _("This is a development version of Grip. If you encounter problems, you are encouraged to revert to the latest stable version."));
     }*/
   }
 
   g_signal_connect(app, "window-state-event", G_CALLBACK(AppWindowStateCB), ginfo);
 
-  LogStatus(ginfo,_("Grip started successfully\n"));
-
   return app;
 }
 
@@ -342,15 +321,12 @@ void GripDie(GtkWidget *widget,gpointer data)
 
   ginfo=(GripInfo *)gtk_object_get_user_data(GTK_OBJECT(widget));
   
-#ifndef GRIPCD
   if(ginfo->ripping_a_disc || ginfo->encoding)
-    gnome_app_ok_cancel_modal((GnomeApp *)ginfo->gui_info.app,
-			      _("Work is in progress.\nReally shut down?"),
-			      ReallyDie,(gpointer)ginfo);
-  else ReallyDie(0,ginfo);
-#else
-  ReallyDie(0,ginfo);
-#endif
+    ReallyDie(ConfirmDialog ((GtkWindow *)ginfo->gui_info.app,
+                             _("Work is in progress.\nReally shut down?")),
+              ginfo);
+  else
+    ReallyDie(0,ginfo);
 }
 
 static void ReallyDie(gint reply,gpointer data)
@@ -361,10 +337,8 @@ static void ReallyDie(gint reply,gpointer data)
 
   ginfo=(GripInfo *)data;
 
-#ifndef GRIPCD
   if(ginfo->ripping_a_disc) KillRip(NULL,ginfo);
   if(ginfo->encoding) KillEncode(NULL,ginfo);
-#endif
 
   if(!ginfo->no_interrupt)
     CDStop(&(ginfo->disc));
@@ -388,96 +362,30 @@ GtkWidget *MakeNewPage(GtkWidget *notebook,char *name)
   return page;
 }
 
-static void MakeStatusPage(GripInfo *ginfo)
+static void DoHelp(GtkWidget *widget, gpointer data)
 {
-  GtkWidget *status_page;
-  GtkWidget *vbox,*vbox2;
-  GtkWidget *notebook;
-  GtkWidget *page;
-  GtkWidget *label;
+  GdkScreen *screen;
+  GError *error = NULL;
 
-  status_page=MakeNewPage(ginfo->gui_info.notebook,_("Status"));
+  screen = gtk_widget_get_screen (widget);
 
-  vbox2=gtk_vbox_new(FALSE,0);
-  notebook=gtk_notebook_new();
+  gtk_show_uri(screen, "ghelp:grip", gtk_get_current_event_time(), &error);
 
-  page=gtk_frame_new(NULL);
+  if (error != NULL) {
+    GtkWidget *d;
+    GtkWidget *toplevel = gtk_widget_get_toplevel (widget);
 
-  vbox=gtk_vbox_new(FALSE,4);
-  gtk_container_border_width(GTK_CONTAINER(vbox),3);
+    d = gtk_message_dialog_new(GTK_IS_WINDOW(toplevel) ? GTK_WINDOW(toplevel) : NULL,
+                               GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                               GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                               "%s", _("Unable to open help file"));
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (d),
+                                              "%s", error->message);
+    g_signal_connect (d, "response", G_CALLBACK (gtk_widget_destroy), NULL);
+    gtk_window_present (GTK_WINDOW (d));
 
-  ginfo->gui_info.status_window=NewStatusWindow(vbox);
-
-  gtk_container_add(GTK_CONTAINER(page),vbox);
-  gtk_widget_show(vbox);
-
-  label=gtk_label_new(_("General"));
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),page,label);
-  gtk_widget_show(page);
-
-
-  page=gtk_frame_new(NULL);
-
-  vbox=gtk_vbox_new(FALSE,4);
-  gtk_container_border_width(GTK_CONTAINER(vbox),3);
-
-  ginfo->gui_info.rip_status_window=NewStatusWindow(vbox);
-
-  gtk_container_add(GTK_CONTAINER(page),vbox);
-  gtk_widget_show(vbox);
-
-  label=gtk_label_new(_("Rip"));
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),page,label);
-  /*  gtk_widget_show(page);*/
-
-
-  page=gtk_frame_new(NULL);
-
-  vbox=gtk_vbox_new(FALSE,4);
-  gtk_container_border_width(GTK_CONTAINER(vbox),3);
-
-  ginfo->gui_info.encode_status_window=NewStatusWindow(vbox);
-
-  gtk_container_add(GTK_CONTAINER(page),vbox);
-  gtk_widget_show(vbox);
-
-  label=gtk_label_new(_("Encode"));
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook),page,label);
-  /*  gtk_widget_show(page);*/
-
-
-  gtk_box_pack_start(GTK_BOX(vbox2),notebook,TRUE,TRUE,0);
-  gtk_widget_show(notebook);
-
-  gtk_container_add(GTK_CONTAINER(status_page),vbox2);
-  gtk_widget_show(vbox2);
-}
-
-void LogStatus(GripInfo *ginfo,char *fmt,...)
-{
-  va_list args;
-  char *buf;
-
-  if(!ginfo->gui_info.status_window) return;
-
-  va_start(args,fmt);
-
-  buf=g_strdup_vprintf(fmt,args);
-
-  va_end(args);
-
-  StatusWindowWrite(ginfo->gui_info.status_window,buf);
-
-  g_free(buf);
-}
-
-static void DoHelp(GtkWidget *widget,gpointer data)
-{
-  char *section;
-
-  section=(char *)data;
-
-  gnome_help_display("grip.xml",section,NULL);
+    g_error_free (error);
+  }
 }
 
 static void MakeHelpPage(GripInfo *ginfo)
@@ -492,45 +400,9 @@ static void MakeHelpPage(GripInfo *ginfo)
   gtk_container_border_width(GTK_CONTAINER(vbox),3);
 
   button=gtk_button_new_with_label(_("Table Of Contents"));
-  gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),NULL);
-  gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
-  gtk_widget_show(button);
-
-  button=gtk_button_new_with_label(_("Playing CDs"));
-  gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"cdplayer");
-  gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
-  gtk_widget_show(button);
-
-  button=gtk_button_new_with_label(_("Ripping CDs"));
-  gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"ripping");
-  gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
-  gtk_widget_show(button);
-
-  button=gtk_button_new_with_label(_("Configuring Grip"));
-  gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"configure");
-  gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
-  gtk_widget_show(button);
-
-  button=gtk_button_new_with_label(_("FAQ"));
-  gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"faq");
-  gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
-  gtk_widget_show(button);
-
-  button=gtk_button_new_with_label(_("Getting More Help"));
-  gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"morehelp");
-  gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
-  gtk_widget_show(button);
-
-  button=gtk_button_new_with_label(_("Reporting Bugs"));
-  gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(DoHelp),(gpointer)"bugs");
-  gtk_box_pack_start(GTK_BOX(vbox),button,FALSE,FALSE,0);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(DoHelp), NULL);
+  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
   gtk_widget_show(button);
 
   gtk_container_add(GTK_CONTAINER(help_page),vbox);
@@ -555,11 +427,7 @@ void MakeAboutPage(GripGUI *uinfo)
   vbox=gtk_vbox_new(TRUE,5);
   gtk_container_border_width(GTK_CONTAINER(vbox),3);
 
-#ifndef GRIPCD
   logo=Loadxpm(GTK_WIDGET(uinfo->app),grip_xpm);
-#else
-  logo=Loadxpm(GTK_WIDGET(uinfo->app),gcd_xpm);
-#endif
 
   gtk_box_pack_start(GTK_BOX(vbox),logo,FALSE,FALSE,0);
   gtk_widget_show(logo);
@@ -574,7 +442,7 @@ void MakeAboutPage(GripGUI *uinfo)
   gtk_box_pack_start(GTK_BOX(vbox2),label,FALSE,FALSE,0);
   gtk_widget_show(label);
 
-  label=gtk_label_new("Copyright\n1998-2005, Mike Oliphant\n2016-2017, Johnny A. Solbu");
+  label=gtk_label_new("Copyright\n1998-2005, Mike Oliphant\n2016-2021, Johnny A. Solbu");
   gtk_widget_set_style(label,uinfo->style_wb);
   gtk_box_pack_start(GTK_BOX(vbox2),label,FALSE,FALSE,0);
   gtk_widget_show(label);
@@ -588,12 +456,12 @@ void MakeAboutPage(GripGUI *uinfo)
 
   hbox=gtk_hbox_new(TRUE,0);
 
-  button=gtk_button_new_with_label("https://sourceforge.net/projects/grip/");
+  button=gtk_button_new_with_label(PACKAGE_BUGREPORT);
   gtk_widget_set_style(button,uinfo->style_dark_grey);
   gtk_widget_set_style(GTK_BIN(button)->child,
 		       uinfo->style_dark_grey);
   gtk_signal_connect(GTK_OBJECT(button),"clicked",
-		     GTK_SIGNAL_FUNC(Homepage),NULL);
+		     GTK_SIGNAL_FUNC(Homepage),uinfo);
   gtk_box_pack_start(GTK_BOX(hbox),button,FALSE,FALSE,0);
   gtk_widget_show(button);
 
@@ -629,9 +497,13 @@ static void MakeStyles(GripGUI *uinfo)
   uinfo->style_dark_grey=MakeStyle(&gdkwhite,color_dark_grey,TRUE);
 }
 
-static void Homepage(void)
+static void Homepage(GtkWidget *widget,gpointer data)
 {
-  system("xdg-open https://sourceforge.net/projects/grip/");
+  GripGUI *uinfo=(GripGUI *)data;
+  if (!gtk_show_uri(gtk_widget_get_screen (widget), PACKAGE_BUGREPORT,
+                    gtk_get_current_event_time(), NULL))
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)uinfo->app,
+               _("Error: Trying to show homepage failed."));
 }
 
 static void LoadImages(GripGUI *uinfo)
@@ -673,7 +545,6 @@ static void LoadImages(GripGUI *uinfo)
   uinfo->play_pix[1]=Loadxpm(uinfo->app,random_xpm);
   uinfo->play_pix[2]=Loadxpm(uinfo->app,playlist_xpm);
 
-#ifndef GRIPCD
   uinfo->rip_pix[0]=Loadxpm(uinfo->app,rip0_xpm);
   uinfo->rip_pix[1]=Loadxpm(uinfo->app,rip1_xpm);
   uinfo->rip_pix[2]=Loadxpm(uinfo->app,rip2_xpm);
@@ -692,7 +563,6 @@ static void LoadImages(GripGUI *uinfo)
   uinfo->smile_pix[5]=Loadxpm(uinfo->app,smile6_xpm);
   uinfo->smile_pix[6]=Loadxpm(uinfo->app,smile7_xpm);
   uinfo->smile_pix[7]=Loadxpm(uinfo->app,smile8_xpm);
-#endif
 }
 
 void GripUpdate(GtkWidget *app)
@@ -742,9 +612,8 @@ void UnBusy(GripGUI *uinfo)
 static void DoLoadConfig(GripInfo *ginfo)
 {
   GripGUI *uinfo=&(ginfo->gui_info);
-  char filename[256];
-  char renamefile[256];
-  char *proxy_env,*tok;
+  char filename[512];
+  char renamefile[520];
   char outputdir[256];
   int confret;
   CFGEntry cfg_entries[]={
@@ -802,7 +671,7 @@ static void DoLoadConfig(GripInfo *ginfo)
   ginfo->use_proxy=FALSE;
   ginfo->use_proxy_env=FALSE;
 
-  strcpy(ginfo->dbserver.name,"freedb.freedb.org");
+  strcpy(ginfo->dbserver.name,"gnudb.gnudb.org");
   strcpy(ginfo->dbserver.cgi_prog,"~cddb/cddb.cgi");
   ginfo->dbserver.port=80;
   ginfo->dbserver.use_proxy=0;
@@ -814,7 +683,7 @@ static void DoLoadConfig(GripInfo *ginfo)
   ginfo->dbserver2.use_proxy=0;
   ginfo->dbserver2.proxy=&(ginfo->proxy_server);
 
-  strcpy(ginfo->discdb_submit_email,"freedb-submit@freedb.org");
+  strcpy(ginfo->discdb_submit_email,"submit@gnudb.org");
   ginfo->db_use_freedb=TRUE;
   *ginfo->user_email='\0';
 
@@ -892,7 +761,7 @@ static void DoLoadConfig(GripInfo *ginfo)
   ginfo->stop_between_tracks=FALSE;
   *ginfo->wav_filter_cmd='\0';
   *ginfo->disc_filter_cmd='\0';
-  ginfo->selected_encoder=6;
+  ginfo->selected_encoder=0;
   strcpy(ginfo->mp3cmdline,"-o %m -a %a -l %d -t %n -b %b %w -N %t -G %G -d %y");
   FindExeInPath("oggenc", ginfo->mp3exename, sizeof(ginfo->mp3exename));
   strcpy(ginfo->mp3fileformat,"~/ogg/%A/%d/%t_%n.%x");
@@ -920,18 +789,18 @@ static void DoLoadConfig(GripInfo *ginfo)
   *ginfo->sprefs.allow_these_chars='\0';
   ginfo->show_tray_icon=TRUE;
 
-  sprintf(filename,"%s/%s",getenv("HOME"),ginfo->config_filename);
+  sprintf(filename,"%s/%s", g_get_home_dir(), ginfo->config_filename);
 
   confret=LoadConfig(filename,"GRIP",2,2,cfg_entries);
 
   if(confret<0) {
     /* Check if the config is out of date */
     if(confret==-2) {
-      gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                        _("Your config file is out of date -- "
-                          "resetting to defaults.\n"
-                          "You will need to re-configure Grip.\n"
-                          "Your old config file has been saved with -old appended."));
+      DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+                 _("Your config file is out of date -- "
+                   "resetting to defaults.\n"
+                   "You will need to re-configure Grip.\n"
+                   "Your old config file has been saved with -old appended."));
 
       sprintf(renamefile,"%s-old",filename);
 
@@ -944,7 +813,6 @@ static void DoLoadConfig(GripInfo *ginfo)
   LoadRipperConfig(ginfo,ginfo->selected_ripper);
   LoadEncoderConfig(ginfo,ginfo->selected_encoder);
 
-#ifndef GRIPCD
   /* Phase out 'outputdir' variable */
 
   if(*outputdir) {
@@ -970,7 +838,6 @@ static void DoLoadConfig(GripInfo *ginfo)
 
     *outputdir='\0';
   }
-#endif
 
   ginfo->dbserver2.use_proxy=ginfo->dbserver.use_proxy=ginfo->use_proxy;
   ginfo->dbserver2.proxy=ginfo->dbserver.proxy;
@@ -978,45 +845,41 @@ static void DoLoadConfig(GripInfo *ginfo)
   ginfo->num_cpu=ginfo->edit_num_cpu;
 
   if(!*ginfo->user_email) {
-    char *host;
-    char *user;
-
-    host = getenv("HOST");
-    if(!host)
-      host = getenv("HOSTNAME");
-    if(!host)
-      host = "localhost";
-
-    user = getenv("USER");
-    if(!user) 
-      user = getenv("USERNAME");
-    if(!user)
-      user = "user";
-
-    g_snprintf(ginfo->user_email,256,"%s@%s",user,host);
+    g_snprintf(ginfo->user_email, 256, "%s@%s", g_get_user_name(), g_get_host_name());
   }
 
   if(ginfo->use_proxy_env) {   /* Get proxy info from "http_proxy" */
-    proxy_env=getenv("http_proxy");
-
+    char * const proxy_env = g_strdup (g_getenv ("http_proxy"));
     if(proxy_env) {
-      
-      /* Skip the "http://" if it's present */
-      
-      if(!strncasecmp(proxy_env,"http://",7)) proxy_env+=7;
-      
-      tok=strtok(proxy_env,":");
-      if(tok) strncpy(ginfo->proxy_server.name,tok,256);
-      
-      tok=strtok(NULL,"/");
-      if(tok) ginfo->proxy_server.port=atoi(tok);
-      
-      Debug(_("server is %s, port %d\n"),ginfo->proxy_server.name,
-	    ginfo->proxy_server.port);
+      LoadProxyConfig(ginfo, proxy_env);
     }
+    g_free (proxy_env);
   }
 }
 
+/** Extract proxy server name and port from proxy_env and update ginfo */
+void LoadProxyConfig(GripInfo * const ginfo, char* proxy_env)
+{
+  g_assert(ginfo);
+  g_assert(proxy_env);
+
+  /* Skip the "http://" if it's present */
+  const char* SCHEME_PFX = "http://";
+  if(!strncasecmp(proxy_env, SCHEME_PFX, strlen(SCHEME_PFX))) proxy_env += strlen(SCHEME_PFX);
+
+  char *tok=strtok(proxy_env,":");
+  if(tok) strncpy(ginfo->proxy_server.name,tok,255);
+
+  /* Make sure ginfo->proxy_server.name is terminated correctly. */
+  ginfo->proxy_server.name[255] = 0;
+
+  tok=strtok(NULL,"/");
+  if(tok) ginfo->proxy_server.port=atoi(tok);
+
+  Debug(_("server is %s, port %d\n"), ginfo->proxy_server.name,
+        ginfo->proxy_server.port);
+}
+
 void DoSaveConfig(GripInfo *ginfo)
 {
   char filename[256];
@@ -1028,11 +891,11 @@ void DoSaveConfig(GripInfo *ginfo)
 
   if(ginfo->edit_num_cpu>MAX_NUM_CPU) ginfo->edit_num_cpu=MAX_NUM_CPU;
 
-  g_snprintf(filename,256,"%s/%s",getenv("HOME"),ginfo->config_filename);
+  g_snprintf(filename,256,"%s/%s", g_get_home_dir(), ginfo->config_filename);
 
   if(!SaveConfig(filename,"GRIP",2,cfg_entries))
-    gnome_app_warning((GnomeApp *)ginfo->gui_info.app,
-                      _("Error: Unable to save config file."));
+    DisplayMsg(GTK_MESSAGE_WARNING, (GtkWindow *)ginfo->gui_info.app,
+               _("Error: Unable to save config file."));
 
   SaveRipperConfig(ginfo,ginfo->selected_ripper);
   SaveEncoderConfig(ginfo,ginfo->selected_encoder);
