@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301
  * USA
  */
 
@@ -43,4 +43,3 @@ gboolean LoadEncoderConfig(GripInfo *ginfo,int encodecfg);
 void SaveEncoderConfig(GripInfo *ginfo,int encodecfg);
 char *FindExe(char *exename,char **paths);
 void FindExeInPath(char *exename, char *buf, int bsize);
-gboolean FileExists(char *filename);
